/**
 * Name        : main.c
 * Version     :
 * Description : main definition for FreeRTOS application
 */

/*
 * FreeRTOS includes
 */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "semphr.h"

/*
 * CMSIS Includes
 */
#include "LPC17xx.h"
#include "lpc17xx_adc.h"
#include "lpc17xx_dac.h"
#include "lpc17xx_pinsel.h"
#include "lpc17xx_gpio.h"

/*
 * defines
 */
#define USERTASK_STACK_SIZE configMINIMAL_STACK_SIZE
#define _700UA 0


void __error__(char *pcFilename, unsigned long ulLine) {
}


/*
 * tareas
 */
void recibir(void *pvParameters);
void enviar(void *pvParameters);
void mostrar(void *pvParameters);

static void setupHardware(void) {
	
	PINSEL_CFG_Type PINSELCfg;
	//ADC
	PINSELCfg.Portnum 	= 	PINSEL_PORT_0;
	PINSELCfg.Funcnum 	= 	PINSEL_FUNC_1;
	PINSELCfg.Pinnum 	=	PINSEL_PIN_23;
	PINSELCfg.Pinmode	=	PINSEL_PINMODE_PULLDOWN;
	PINSEL_ConfigPin(&PINSELCfg);
	//DAC
	PINSELCfg.Pinnum	=	PINSEL_PIN_26;
	PINSELCfg.Funcnum	=	PINSEL_FUNC_2;
	PINSELCfg.Pinmode	=	PINSEL_PINMODE_PULLUP;
	PINSEL_ConfigPin(&PINSELCfg);
	//GPIO
	GPIO_SetDir(0,0xFF,1);
}

/*
 * handles
 */

xTaskHandle 	recibirCtrl;
xTaskHandle 	enviarCtrl;
xTaskHandle		mostarCtrl;
xQueueHandle 	QueueCtrl;


/**
 * Program entry point 
 */
int main(void) {
	setupHardware();

	xTaskCreate(recibir,(signed char *)"recibe datos del ADC",USERTASK_STACK_SIZE,NULL,5,&recibirCtrl);
	xTaskCreate(enviar,(signed char *)"envía datos al DAC",USERTASK_STACK_SIZE,NULL,4,&enviarCtrl);
	xTaskCreate(mostrar,(signed char *)"muestra el valor del ADC",USERTASK_STACK_SIZE,NULL,3,&mostarCtrl);

	QueueCtrl = xQueueCreate(10,sizeof(int16_t));

	vTaskStartScheduler();

	/* 
	 * Will only get here if there was insufficient memory to create the idle task. 
	 */
	return 1;
}

void recibir(void *pvParameters) {
	int16_t dato=0x00;
	portBASE_TYPE EdoQueue;
	unsigned portBASE_TYPE prioridad;

	ADC_Init(LPC_ADC,200000);
	ADC_ChannelCmd(LPC_ADC,0,ENABLE);
	for(;;) {
		ADC_StartCmd(LPC_ADC,ADC_START_NOW);
		dato=ADC_ChannelGetData(LPC_ADC,ADC_CHANNEL_0);
		EdoQueue= xQueueSend(QueueCtrl,&dato,200/portTICK_RATE_MS);
		if(EdoQueue==!pdPASS) {
			prioridad=uxTaskPriorityGet(NULL);
			if(prioridad>3)
				vTaskPrioritySet(NULL,(prioridad-1)); }
		vTaskDelay(100/portTICK_RATE_MS);} }

void enviar(void *pvParemeters) {
	int16_t dato=0x00;
	portBASE_TYPE EdoQueue;
	unsigned portBASE_TYPE prioridad;

	DAC_Init(LPC_DAC);
	DAC_SetBias(LPC_DAC,_700UA);
	for(;;) {
		EdoQueue=xQueueReceive(QueueCtrl,&dato,200/portTICK_RATE_MS);
		if(EdoQueue == pdPASS) {
			dato/=4;
			DAC_UpdateValue(LPC_DAC,dato); }
		else {
			prioridad=uxTaskPriorityGet(recibirCtrl);
			if(prioridad<5)
				vTaskPrioritySet(NULL,(prioridad+1)); }
	vTaskDelay(200/portTICK_RATE_MS);} }

void mostrar(void *pvParameters) {
	int16_t dato=0x00;
	portBASE_TYPE EdoQueue;

	for(;;) {
		EdoQueue = xQueuePeek(QueueCtrl,&dato,0);
		if(EdoQueue==pdPASS) {
			GPIO_ClearValue(0,0xFF);
			dato/=0x10;
			GPIO_SetValue(0,dato); }
		vTaskDelay(500/portTICK_RATE_MS); } }















